import 'app_exception.dart';

class AppEvent<In, Out> {
  final List<Out Function(In? data)> _callbacks = [];

  void subscribe(Out Function(In? data) callback) {
    _callbacks.add(callback);
  }

  void unsubscribe(Out Function(In? data) callback) {
    _callbacks.remove(callback);
  }

  void invoke([In? data]) {
    for (var callback in _callbacks) {
      callback.call(data);
    }
  }
}

class AppMonoEvent<In, Out> {
  final List<Out Function(In? data)> _callbacks = [];

  void subscribe(Out Function(In? data) callback) {
    if (_callbacks.isEmpty) {
      _callbacks.add(callback);
    } else {
      throw AppException("AppDirectEvent already has a subscriber.");
    }
  }

  void unsubscribe(Out Function(In? data) callback) {
    _callbacks.remove(callback);
  }

  Out invoke([In? data]) {
    if (_callbacks.isEmpty) {
      throw AppException("AppDirectEvent was invoked and has no subscriber.");
    }
    return _callbacks.first.call(data);
  }
}

class AppListEvent<In, Out> {
  final List<Out Function(In? data)> _callbacks = [];

  void subscribe(Out Function(In? data) callback) {
    _callbacks.add(callback);
  }

  void unsubscribe(Out Function(In? data) callback) {
    _callbacks.remove(callback);
  }

  List<Out> invoke([In? data, List<Out>? list]) {
    list ??= <Out>[];
    for (var callback in _callbacks) {
      var out = callback.call(data);
      list.add(out);
    }
    return list;
  }
}
