import 'package:flutter/material.dart';
import 'package:modular_system_core/modular_system_core.dart';
import 'package:modular_system_core/src/widget/entity_table/table_extra_options.dart';
import 'package:modular_system_core/src/widget/entity_table/table_tile.dart';

class EntityTableBody<T> extends StatefulWidget {
  // ignore: use_key_in_widget_constructors
  const EntityTableBody({
    required this.appViewModel,
    required this.viewModel,
  });

  final AppViewModel appViewModel;
  final EntityTableViewModel<T> viewModel;

  @override
  State<EntityTableBody> createState() {
    // ignore: no_logic_in_create_state
    return _EntityTableBodyState<T>(
      appViewModel: appViewModel,
      viewModel: viewModel,
    );
  }
}

class _EntityTableBodyState<T> extends State<EntityTableBody> {
  _EntityTableBodyState({
    required this.appViewModel,
    required this.viewModel,
  }) {
    subscribeToEvents();
    setUpdateFuture();
  }

  @override
  void dispose() {
    unsubscribeToEvents();
    super.dispose();
  }

  final AppViewModel appViewModel;
  final EntityTableViewModel<T> viewModel;
  late Future updateTableFuture;
  late List<Widget> listData = [];
  bool selectorOn = false;

  void subscribeToEvents() {
    viewModel.events.updateEvent.subscribe(onUpdate);
  }

  void unsubscribeToEvents() {
    viewModel.events.updateEvent.unsubscribe(onUpdate);
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      color: appViewModel.colorScheme.background,
      child: CustomScrollView(
        slivers: [
          EntityTableAppBar(
            viewModel: viewModel,
            appViewModel: appViewModel,
            updateTableFuture: updateTableFuture,
          ),
          FutureBuilder(
            future: updateTableFuture,
            builder: (context, snapshot) {
              if (snapshot.connectionState == ConnectionState.done) {
                if (snapshot.hasError) {
                  return SliverToBoxAdapter(
                    child: Text(
                      "ERRO",
                      style: appViewModel.errorTextStyle(AppTextType.subtitle),
                    ),
                  );
                }
                return SliverList(
                  key: UniqueKey(),
                  delegate: SliverChildListDelegate(listData),
                );
              }
              return SliverToBoxAdapter(
                child: appViewModel.loading(),
              );
            },
          ),
        ],
      ),
    );
  }

  // Event Handlers
  void onUpdate(dynamic data) {
    setState(() {
      setUpdateFuture();
    });
  }

  void setUpdateFuture() {
    updateTableFuture = Future(() async {
      try {
        var map = await viewModel.properties.update(viewModel.state.search);
        viewModel.state.paginationOut = map.key;
        viewModel.state.data = map.value;
        setListData();
      } catch (e) {
        appViewModel.appEvents.errorEvent.invoke(e);
      }
    });
  }

  void setListData() {
    listData.clear();
    listData.add(TableExtraOptions(
      appViewModel: appViewModel,
      viewModel: viewModel,
      child: viewModel.properties.extraOptions,
    ));
    for (var index = 0; index < viewModel.state.data.length; index++) {
      listData.add(TableTile(
        index: index,
        appViewModel: appViewModel,
        viewModel: viewModel,
        child: viewModel.properties.itemBuilder(
          context,
          viewModel.state.data[index],
          viewModel,
        ),
      ));
    }
  }
}
